#R Code for SN Chapman & V Lummaa "Grandmother Effects Over the Finnish Demographic Transition"
#Methodological details can be found in Methods section

#if you have any queries about the study, please email Simon N. Chapman at sinich@utu.fi
#this file presents code for reanalysis - datasets for this purpose are available as supplementary files with the manuscript
#for data requests beyond these supplementary files, please contact Virpi Lummaa at virpi.lummaa@utu.fi to discuss possible collaboration

#Please note that mixed() may require computer to have over 8GB of RAM to successfully run
#Computation times can be quite long too

#load in required packages
library(lme4)
library(afex)
library(muhaz)

#1 - Hazards
#read in hazard rate data - remember to set working directory to appropriate location
DATA_S1 <- read.csv("Supplementary Information - Hazard Rate Data.csv", stringsAsFactors = FALSE)

#dataset is one row per individual (within time period, so cols 1-3 may be different individual to cols 4-6 and 7-8), giving age, uncensored age (in the case of pre-industrial and transitional - see Methods and Figure S1 for details), and survival status (1 = died, 0 = censored)

#pre-industrial age
#partition data into preindustrial, transitional, post-transitional
preindHaz <- DATA_S1[1:19270, 1:3]
transitHaz <- DATA_S1[, 4:6]
posttransHaz <- DATA_S1[1:18852, 7:8]

#use kphaz.fit from muhaz, using age and survival status
kp1 <- kphaz.fit(preindHaz[[1]], preindHaz[[3]])
kp2 <- kphaz.fit(transitHaz[[1]], transitHaz[[3]])
kp3 <- kphaz.fit(posttransHaz[[1]], posttransHaz[[2]])

#Cumulative hazards can be obtained by applying cumsum to the hazards
#for example:
cumsum(kp1[["haz"]])

#2 - Survival
#read in survival data
DATA_S2 <- read.csv("Supplementary Information - Survival Data.csv", stringsAsFactors = FALSE)

#dataset is for discrete time-event analyses
#one row per individual per year

#variables are as follows:
#survstatus - survival of individual in given year, 1 = alive, 0 = dead
#age - age of the focal individual
#mgmstatus - maternal grandmother survial status a.dead or b.alv (alive)
#timeperiod - 3-level factor for the time period the observation is from; per-industrial, transitional, post-transition
#bcohort - birth cohort; factor with decade of birth; to be used as random effect
#mumstatus - mother survival status - 1 for alive, 0 for dead
#sibs15 - number of living siblings under the age of 15 (potential helpers/competitors)
#twin - whether the child was born a twin (2) or not (1)
#mgmid - maternal grandmother ID
#mumid - mother ID
#bcohort2 - numeric (continuous) version of bcohort variable
#ncous - number of cousins under age 15

#time period model
cttMod <- glmer(survstatus ~ age + mgmstatus * timeperiod + mumstatus + sibs15 + twin + ncous + (1|mgmid/mumid) + (1|bcohort), data = DATA_S2, family = binomial(link = "logit"), control = glmerControl(optimizer = "nloptwrap", optCtrl = list(maxfun = 250000)))
#testing the significance of the interaction term with 'mixed' from afex 
cttModInt <- mixed(survstatus ~ age + mgmstatus * timeperiod + mumstatus + sibs15 + twin + ncous + (1|mgmid/mumid) + (1|bcohort), data = DATA_S2, family = binomial(link = "logit"), control = glmerControl(optimizer = "nloptwrap", optCtrl = list(maxfun = 250000)), method = "LRT")

#continuous birth cohort instead of 3-level time period
cttModAltBC <- glmer(survstatus ~ age + mgmstatus * bcohort2 + mumstatus + sibs15 + twin + ncous + (1|mgmid/mumid), data = DATA_S2, family = binomial(link = "logit"), control = glmerControl(optimizer = "nloptwrap", optCtrl = list(maxfun = 250000)))
#testing the significance of the interaction term with 'mixed' from afex
cttModIntAltBC <- mixed(survstatus ~ age + mgmstatus * bcohort2 + mumstatus + sibs15 + twin + ncous + (1|mgmid/mumid), data = DATA_S2, family = binomial(link = "logit"), control = glmerControl(optimizer = "nloptwrap", optCtrl = list(maxfun = 250000)), method = "LRT")

#3 - Fertility
#read in fertility data
DATA_S3 <- read.csv("Supplementary Information - Fertility Data.csv", stringsAsFactors = FALSE)

#dataset is for discrete time-event analysis
#as above, one row per individual per year, though the dataset needs to be split in two - one part for age at first birth, one for birth spacing. Within in one row, then, there may be different individuals/the same individuals in different years

#first take the data for age at first birth
S3_AgeFirst <- DATA_S3[c(1:20193), c(1:9)]
#variables:
#birth - whether focal individual gave birth in year; 0 for no birth or 1 for birth
#age - age of focal individual
#age2 - quadratic age of focal individual
#mumtype - indicator of F0 generation; both dead, mother alive, mother-in-law alive, both alive
#timeperiod1 - time period; 3-level factor: pre-industrial, transitional, post-transition
#bOrder - birth order of the focal individual
#living_sibs - number of reproductive age sisters
#mumid - mother id (i.e. maternal grandmother of the child born)
#bcohort - numeric version of birthing cohort (decade in which individual had their first child)

#Age at first birth, using time period
afbMod <- glmer(birth ~ age * mumtype * timeperiod + age2 + bOrder + living_sibs + (1|mumid), data = S3_AgeFirst, family = binomial(link = 'logit'), control = glmerControl(optimizer = 'nloptwrap', optCtrl = list(maxfun = 150000)))
#mixed function to get significance of interaction
afbModMix <- mixed(birth ~ age * mumtype * timeperiod + age2 + bOrder + living_sibs + (1|mumid), data = S3_AgeFirst, family = binomial(link = 'logit'), control = glmerControl(optimizer = 'nloptwrap', optCtrl = list(maxfun = 250000)), method = 'LRT')

#Age at first birth, using birth cohort (continuous)
afbMod2 <- glmer(birth ~ age * mumtype * bcohort + age2 + bOrder + living_sibs + (1|mumid), data = S3_AgeFirst, family = binomial(link = 'logit'), control = glmerControl(optimizer = 'nloptwrap', optCtrl = list(maxfun = 150000)))

afbModMixAlt <- mixed(birth ~ age * mumtype * bcohort + age2 + bOrder + living_sibs + (1|mumid), data = S3_AgeFirst, family = binomial(link = 'logit'), control = glmerControl(optimizer = 'nloptwrap', optCtrl = list(maxfun = 250000)), method = 'LRT')

#Birth spacing
S3_BirthSpace <- DATA_S3[, c(10:19)]
#variables:
#birth.1 - whether focal individual gave birth in year; 0 for no birth or 1 for birth
#age.1 - age of focal individual
#timeLastBir - time since last birth, capped at 10 (see Methods for details)
#mumtype.1 - indicator of F0 generation; both dead, mother alive, mother-in-law alive, both alive
#timeperiod.1 - time period; 3-level factor: pre-industrial, transitional, post-transition
#prevKid - survival status of previous child; 1 = alive, 0 = dead
#kidBord - birth order of the (hypothetical) next child
#ID - individual ID
#mumid.1 - mother id (i.e. maternal grandmother of the child born)
#living_sibs.1 - number of reproductive age sisters

#timeperiod model
ibiMod <- glmer(birth.1 ~ age.1 + timeLastBir*mumtype.1*timeperiod.1 + timeLastBir*prevkid + kidBord + living_sibs.1 + (1|mumid.1/ID), data = S3_BirthSpace, family = binomial (link = 'logit'), control = glmerControl(optimizer = 'nloptwrap', optCtrl = list(maxfun = 150000)))
#mixed for testing significance of interaction
ibiModMix <- mixed(birth.1 ~ age.1 + timeLastBir*mumtype.1*timeperiod.1 + timeLastBir*prevkid + kidBord + living_sibs.1 + (1|mumid.1/ID), data = S3_BirthSpace, family = binomial (link = 'logit'), control = glmerControl(optimizer = 'nloptwrap', optCtrl = list(maxfun = 150000)), method = "LRT")
